<?php
/**
 * 2007-2022 ETS-Soft
 *
 * NOTICE OF LICENSE
 *
 * This file is not open source! Each license that you purchased is only available for 1 website only.
 * If you want to use this file on more websites (or projects), you need to purchase additional licenses.
 * You are not allowed to redistribute, resell, lease, license, sub-license or offer our resources to any third party.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author ETS-Soft <etssoft.jsc@gmail.com>
 * @copyright  2007-2022 ETS-Soft
 * @license    Valid for 1 website (or project) for each purchase of license
 *  International Registered Trademark & Property of ETS-Soft
 */

if (!defined('_PS_VERSION_'))
	exit;

class Create_discount
{

	private $id;
	private $id_currency;
	private $reduction_percent;
	private $reduction_amount;
	private $reduction_tax;
	private $day;
	private $name;
	private $free_shipping;
	private $apply_discount;
	private $discount_code;
	private $discount_option;
	private $context;
	private $date_from;
	private $date_to;
	private $exclude_products;
//	private $allow_multi_discount;

	public function __construct($id_discount = null)
	{
		if ($id_discount) {
			$this->id = $id_discount;
		}
	}

	public function createDiscount($name = null)
	{

		if (!$this->getConfig()) {
			return false;
		}

		$language = Language::getLanguages(true);
		$cart_rule = new CartRule();
		//$cart_rule -> id_customer = $id_customer;
		if (is_array($language) && $language) {
			$ruler_name = array();
			foreach ($language as $lang) {
				$ruler_name[$lang['id_lang']] = $name ? $name : 'dev';
			}
			$cart_rule->name = $ruler_name;
		}
		$cart_rule->free_shipping = (bool)$this->free_shipping;

		if ($this->discount_option == '2') {
			$cart_rule->code = $this->discount_code;
			$cart_rule_temp = CartRule::getCartsRuleByCode($this->discount_code, Context::getContext()->language->id);
			$this->id = $cart_rule_temp[0]['id_cart_rule'];
			$this->id_currency = $cart_rule_temp[0]['reduction_currency'];
			$this->reduction_percent = $cart_rule_temp[0]['reduction_percent'];
			$this->reduction_amount = $cart_rule_temp[0]['reduction_amount'];
			$this->reduction_tax = $cart_rule_temp[0]['reduction_tax'];
			$this->date_from = $cart_rule_temp[0]['date_from'];
			$this->date_to = $cart_rule_temp[0]['date_to'];
		} else {
			do {
				$prefix = Configuration::get('YBC_NEWSLETTER_DISCOUNT_PREFIX');
				if ($prefix) {
					$code = self::genCode(8, $prefix);
				} else {
					$code = self::genCode(8);
				}
				$cart_rule->code = $code;
				$cart_rule->reduction_exclude_special = 1;
			} while (CartRule::cartRuleExists($code));
		}
		$cart_rule->reduction_currency = (int)$this->id_currency;
		$cart_rule->reduction_percent = (float)$this->reduction_percent;
		$cart_rule->reduction_amount = (float)$this->reduction_amount;
		$cart_rule->reduction_tax = (bool)$this->reduction_tax;
		$cart_rule->date_from = $this->date_from ? $this->date_from : date('Y-m-d H:i:s');
		$cart_rule->date_to = $this->date_to ? $this->date_to : ((int)$this->day ? date('Y-m-d H:i:s', strtotime('+' . (int)$this->day . ' days')) : date('Y-m-d H:i:s', strtotime('+30 days')));
		$cart_rule->reduction_exclude_special = $this->exclude_products;

		if ($this->discount_option == '2' && $this->checkVoucher($name, $this->discount_code)) {
			$cart_rule->id = $this->id;
			return $cart_rule;
		} else if ($cart_rule->add()) {
			return $cart_rule;
		}
		return false;
	}

	public function getConfig()
	{
		if (!$this->context) {
			$this->context = Context::getContext();
		}

		$this->discount_option = Configuration::get('YBC_NEWSLETTER_DISCOUNT_OPTION');
		$this->discount_code = Configuration::get('YBC_NEWSLETTER_DISCOUNT_CODE');
		$this->id_currency = Configuration::get('YBC_NEWSLETTER_DISCOUNT_ID_CURRENCY');
		$this->reduction_percent = Configuration::get('YBC_NEWSLETTER_DISCOUNT_REDUCTION_PERCENT');
		$this->reduction_amount = Configuration::get('YBC_NEWSLETTER_DISCOUNT_REDUCTION_AMOUNT');
		$this->reduction_tax = Configuration::get('YBC_NEWSLETTER_DISCOUNT_REDUCTION_TAX');
		$this->day = Configuration::get('YBC_NEWSLETTER_DISCOUNT_APPLY_DISCOUNT_IN');
		$this->free_shipping = Configuration::get('YBC_NEWSLETTER_DISCOUNT_FREE_SHIPPING');
		$this->apply_discount = Configuration::get('YBC_NEWSLETTER_DISCOUNT_APPLY_DISCOUNT');
		$this->exclude_products = Configuration::get('YBC_NEWSLETTER_DISCOUNT_EXCLUDE_PRODUCTS');
//		$this->allow_multi_discount = Configuration::get('YBC_NEWSLETTER_ALLOW_MULTI_DISCOUNT');

		if ($this->apply_discount === 'percent') {
			$this->reduction_amount = $this->reduction_tax = $this->id_currency = '';
		} elseif ($this->apply_discount === 'amount') {
			$this->reduction_percent = '';
		} else {
			$this->reduction_amount = $this->reduction_tax = $this->id_currency = $this->reduction_percent = '';
		}

		return true;
	}

	public static function genCode($size, $prefix = '')
	{
		$chars = "123456789ABCDEFGHIJKLMNPQRSTUVWXYZ";
		$code = '';
		for ($i = 1; $i <= $size; ++$i)
			$code .= Tools::substr($chars, rand(0, Tools::strlen($chars) - 1), 1);
		return $prefix . $code;
	}

	public function checkVoucher($name = null, $code = null, $id_shop = null, $id_lang = null)
	{
		if (!$name || !$code) {
			return false;
		}
		if (!$id_shop) {
			$id_shop = Context::getContext()->shop->id;
		}
		if (!$id_lang) {
			$id_lang = Context::getContext()->language->id;
		}

		$sql = 'SELECT cr.id_cart_rule
						FROM ' . _DB_PREFIX_ . 'cart_rule cr
						LEFT JOIN ' . _DB_PREFIX_ . 'cart_rule_lang crl ON (cr.id_cart_rule = crl.id_cart_rule) 
                        WHERE cr.`code` = \'' . pSQL($code) . '\' 
                        AND crl.`name` = \'' . pSQL($name) . '\'
                        AND crl.`id_lang` = \'' . (int)$id_lang . '\' ';

		return (bool)Db::getInstance()->getValue($sql);
	}

	public function deleteDisCount()
	{
		$this->id = CartRule::getIdByCode($this->code);
		$cart_rule = new CartRule($this->id);
		return (bool)$cart_rule->delete();
	}
}